<?php

/**
 * Vvveb
 *
 * Copyright (C) 2022  Ziadin Givan
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 */

/**
 * @package SEO Plugin
 * @version 0.1
 */
/*
Name: SEO Optimization
Slug: seo
Url: https://www.vvveb.com
Description: Add SEO capabilities like meta for content.
Author: givanz
Version: 0.1
Thumb: seo.svg
Author url: https://www.vvveb.com
Settings: /admin/index.php?module=plugins/seo/settings
*/

use function Vvveb\__;
use function Vvveb\getMultiPostContentMeta;
use function Vvveb\getMultiSettingContent;
use function Vvveb\isEditor;
use function Vvveb\setMultiPostContentMeta;
use function Vvveb\setMultiSettingContent;
use Vvveb\System\Core\Request;
use Vvveb\System\Core\View;
use Vvveb\System\Event;

if (! defined('V_VERSION')) {
	die('Invalid request!');
}

class SeoPlugin {
	private $view;

	function getPost($post, $post_id) {
		$this->view->seo = $this->view->seo ?? [];

		if ($post_id) {
			$seo = getMultiPostContentMeta($post_id, 'seo') ?? [];

			foreach ($seo as $meta) {
				$this->view->seo[$meta['language_id']][$meta['key']] = $meta['value'];
			}
		}

		return [$post, $post_id];
	}

	function getSite($site, $setting, $site_id, $data) {
		$this->view->seo = $this->view->seo ?? [];

		$seo = getMultiSettingContent($site_id, 'seo') ?? [];

		foreach ($seo as $meta) {
			$this->view->seo[$meta['language_id']][$meta['key']] = $meta['value'];
		}

		return [$site, $setting, $site_id, $data];
	}

	function post($content, $language, $slug) {
		$post        = $content[$language] ?? [];
		$post_id     = $post['post_id'] ?? false;
		$language_id = $post['language_id'] ?? false;

		if ($post_id) {
			$seo  = [];
			$meta = getMultiPostContentMeta($post_id, 'seo', null, [], $language_id) ?? [];

			foreach ($meta as $item) {
				$seo[$item['key']] = $item['value'];
			}

			$this->view->seo['meta'] = $seo;
		}
		/*
		$this->view->seo                    = ['json-schema-graph' => $post['name'] ?? ''];
		$this->view->seo['meta']['og']      = ['json-schema-graph' => $post['name'] ?? ''];
		$this->view->seo['meta']['article'] = ['json-schema-graph' => $post['name'] ?? ''];
		$this->view->seo['meta']['twitter'] = ['json-schema-graph' => $post['name'] ?? ''];
		$this->view->seo['meta']['itemprop'] = ['inLanguage' => 'en'];
		*/

		return [$content, $language, $slug];
	}

	function save($post, $post_id, $type) {
		//var_dump($post);
		$request = Request::getInstance();
		$seo     = $request->post['seo'] ?? [];

		$meta = [];

		foreach ($seo as $key => $values) {
			foreach ($values as $language_id => $value) {
				$meta[] = ['namespace' => 'seo', 'key' => $key, 'value' => $value, 'language_id' => $language_id];
			}
		}

		setMultiPostContentMeta($post_id, $meta);

		return [$post, $post_id, $type];
	}

	function saveSite($site, $settings, $site_id, $data) {
		//var_dump($post);
		$request = Request::getInstance();
		$seo     = $request->post['seo'] ?? [];
		//var_dump($seo);

		$meta = [];

		foreach ($seo as $key => $values) {
			foreach ($values as $language_id => $value) {
				$meta[] = ['namespace' => 'seo', 'key' => $key, 'value' => $value, 'language_id' => $language_id];
			}
		}

		setMultiSettingContent($site_id, $meta);

		return [$site, $settings, $site_id, $data];
	}

	function addSeoTabs() {
		//add script on compile
		Event::on('Vvveb\System\Core\View', 'compile', __CLASS__, function ($template, $htmlFile, $tplFile, $vTpl, $view) {
			//insert js and css on post and product page
			if ($template == 'content/post.html' || $template == 'product/product.html' || $template == 'settings/site.html') {
				//insert script
				$vTpl->loadTemplateFile(__DIR__ . '/admin/template/seotab.tpl', true);
				//$vTpl->addCommand('body|append', $script);
			}

			return [$template, $htmlFile, $tplFile, $vTpl, $view];
		});
	}

	function admin() {
		$this->addSeoTabs();

		//add admin menu item
		$admin_path = \Vvveb\adminPath();
		Event::on('Vvveb\Controller\Base', 'init-menu', __CLASS__, function ($menu) use ($admin_path) {
			$menuItem = [
				'name'     => __('SEO'),
				'url'      => $admin_path . 'index.php?module=plugins/seo/settings',
				'module'   => 'plugins/seo/settings',
				//'icon-img' => PUBLIC_PATH . 'plugins/seo/seo.svg',
				'icon' => 'icon-search-outline',
			];

			$menu['plugins']['items']['seo-plugin'] = $menuItem;
			$menu['seo-plugin'] = $menuItem;

			return [$menu];
		});

		Event::on('Vvveb\Controller\Content\Edit', 'save', __CLASS__, [$this, 'save']);
		Event::on('Vvveb\Controller\Content\Edit', 'index', __CLASS__, [$this, 'getPost']);
		Event::on('Vvveb\Controller\Settings\Site', 'index', __CLASS__, [$this, 'getSite']);
		Event::on('Vvveb\Controller\Settings\Site', 'save', __CLASS__, [$this, 'saveSite']);
	}

	function app() {
		if (isEditor()) {
			return;
		}
		$template = $this->view->getTemplateEngineInstance();
		$template->loadTemplateFile(__DIR__ . '/app/template/common.tpl');

		$this->view->seo = \Vvveb\getSetting('seo');

		//Event::on('Vvveb\Component\Post', 'results', __CLASS__, [$this,'post']);
		Event::on('Vvveb\Controller\Content\Post', 'index', __CLASS__, [$this, 'post']);
	}

	function __construct() {
		$this->view     = View::getInstance();

		if (APP == 'admin') {
			$this->admin();
		} else {
			if (APP == 'app') {
				$this->app();
			}
		}
	}
}

$seoPlugin = new SeoPlugin();
